@@ -1,10 +1,15 @@
-
 #ifndef _GABBLE_EXTENSIONS_H
 #define _GABBLE_EXTENSIONS_H
 
-#include "extensions/_gen/svc.h"
 #include "extensions/_gen/enums.h"
+#include "extensions/_gen/svc.h"
+
+G_BEGIN_DECLS
+
+#include "extensions/_gen/gtypes.h"
 #include "extensions/_gen/interfaces.h"
 
+G_END_DECLS
+
 #endif /* _GABBLE_EXTENSIONS_H */
 
