@@ -1,6 +1,10 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "wocky-pubsub-test-helpers.h"
 
-#include <wocky/wocky-pubsub-node.h>
+#include <wocky/wocky.h>
 
 void
 test_pubsub_add_subscription_nodes (
