@@ -1,3 +1,7 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 
 #include "wocky-test-sasl-auth-server.h"
@@ -5,9 +9,7 @@
 #include "wocky-test-stream.h"
 #include "wocky-test-helper.h"
 
-#include <wocky/wocky-xmpp-connection.h>
-#include <wocky/wocky-sasl-auth.h>
-#include <wocky/wocky-auth-registry.h>
+#include <wocky/wocky.h>
 
 typedef struct {
   gchar *description;
