@@ -16,6 +16,9 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
+#if !defined (WOCKY_H_INSIDE) && !defined (WOCKY_COMPILATION)
+# error "Only <wocky/wocky.h> can be included directly."
+#endif
 
 #ifndef __WOCKY_UTILS_H__
 #define __WOCKY_UTILS_H__
@@ -64,7 +67,7 @@ const gchar *wocky_enum_to_nick (GType enum_type, gint value);
 
 gchar *wocky_absolutize_path (const gchar *path);
 
-GList *wocky_list_deep_copy (GBoxedCopyFunc copy, const GList *items);
+GList *wocky_list_deep_copy (GBoxedCopyFunc copy, GList *items);
 
 GString *wocky_g_string_dup (const GString *str);
 
@@ -129,6 +132,18 @@ void wocky_g_string_free (GString *str);
       return NULL; \
     } G_STMT_END
 
+#define wocky_implement_finish_return_pointer(source, tag) \
+    G_STMT_START { \
+      GSimpleAsyncResult *_simple; \
+      _simple = (GSimpleAsyncResult *) result; \
+      if (g_simple_async_result_propagate_error (_simple, error)) \
+        return NULL; \
+      g_return_val_if_fail (g_simple_async_result_is_valid (result, \
+              G_OBJECT (source), tag), \
+          NULL); \
+      return g_simple_async_result_get_op_res_gpointer (_simple); \
+    } G_STMT_END
+
 G_END_DECLS
 
 #endif /* #ifndef __WOCKY_UTILS_H__ */
