@@ -12,6 +12,8 @@ from caps_helper import check_caps, disco_caps
 import constants as cs
 import ns
 
+from config import VOIP_ENABLED
+
 def run_test(q, bus, conn, stream):
     initial_presence = q.expect('stream-presence')
 
@@ -19,13 +21,18 @@ def run_test(q, bus, conn, stream):
 
     # For some reason, until we advertise any capabilities, these caps turn
     # up in our presence
-    check_caps(namespaces, [
-        ns.JINGLE,
-        ns.JINGLE_015,
-        ns.JINGLE_TRANSPORT_ICEUDP,
-        ns.JINGLE_TRANSPORT_RAWUDP,
-        ns.GOOGLE_P2P
-        ])
+    if VOIP_ENABLED:
+        caps = [
+            ns.JINGLE,
+            ns.JINGLE_015,
+            ns.JINGLE_TRANSPORT_ICEUDP,
+            ns.JINGLE_TRANSPORT_RAWUDP,
+            ns.GOOGLE_P2P
+            ]
+    else:
+        caps = []
+
+    check_caps(namespaces, caps)
 
 if __name__ == '__main__':
     exec_test(run_test)
