@@ -9,10 +9,6 @@ import constants as cs
 import sys
 
 def test(q, bus, conn, stream):
-    conn.Connect()
-    q.expect('dbus-signal', signal='StatusChanged',
-             args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
-
     # server closes its stream
     stream.transport.loseConnection()
 
