@@ -2,6 +2,12 @@ import dbus
 import constants as cs
 from file_transfer_helper import SendFileTest, exec_file_transfer_test
 
+from config import FILE_TRANSFER_ENABLED
+
+if not FILE_TRANSFER_ENABLED:
+    print "NOTE: built with --disable-file-transfer"
+    raise SystemExit(77)
+
 class SendFileTransferProvideImmediately(SendFileTest):
     def provide_file(self):
 
@@ -15,7 +21,8 @@ class SendFileTransferProvideImmediately(SendFileTest):
         else:
             assert False
 
-        SendFileTest.provide_file(self)
+        if SendFileTest.provide_file(self):
+            return True
 
         # state is still Pending as remote didn't accept the transfer yet
         state = self.ft_props.Get(cs.CHANNEL_TYPE_FILE_TRANSFER, 'State')
