@@ -3,6 +3,12 @@ import dbus
 import constants as cs
 from file_transfer_helper import SendFileTest, exec_file_transfer_test
 
+from config import FILE_TRANSFER_ENABLED
+
+if not FILE_TRANSFER_ENABLED:
+    print "NOTE: built with --disable-file-transfer"
+    raise SystemExit(77)
+
 class SendFileTransferToUnknownContactTest(SendFileTest):
     def __init__(self, bytestream_cls, file, address_type, access_control, acces_control_param):
         SendFileTest.__init__(self, bytestream_cls, file, address_type, access_control, acces_control_param)
