@@ -11,26 +11,38 @@ import constants as cs
 
 from jingletest2 import JingleProtocol031, JingleTest2
 
+from config import VOIP_ENABLED
+
+if not VOIP_ENABLED:
+    print "NOTE: built with --disable-voip"
+    raise SystemExit(77)
+
 def test(q, bus, conn, stream):
-    remote_jid = 'foo@bar.com/Foo'
+    worker(q, bus, conn, stream, remote_jid='foo@bar.com/Foo')
+
+def test_bare_jid(q, bus, conn, stream):
+    worker(q, bus, conn, stream, remote_jid='foo@sip.bar.com')
+
+def worker(q, bus, conn, stream, remote_jid):
     jp = JingleProtocol031()
     jt2 = JingleTest2(jp, conn, q, stream, 'test@localhost', remote_jid)
     jt2.prepare()
 
     self_handle = conn.GetSelfHandle()
-    remote_handle = conn.RequestHandles(cs.HT_CONTACT, [remote_jid])[0]
+    remote_handle = conn.RequestHandles(cs.HT_CONTACT, [jt2.peer])[0]
 
     # Remote end calls us
     node = jp.SetIq(jt2.peer, jt2.jid, [
         jp.Jingle(jt2.sid, jt2.peer, 'session-initiate', [
-            jp.Content('audiostream', 'initiator', 'both', [
+            jp.Content('audiostream', 'initiator', 'both',
                 jp.Description('audio', [
-                    jp.PayloadType(name, str(rate), str(id)) for
-                        (name, id, rate) in jt2.audio_codecs ]),
-            jp.TransportGoogleP2P() ]) ]) ])
+                    jp.PayloadType(name, str(rate), str(id), parameters) for
+                        (name, id, rate, parameters) in jt2.audio_codecs ]),
+            jp.TransportGoogleP2P()) ]) ])
     stream.send(jp.xml(node))
 
-    nc = q.expect('dbus-signal', signal='NewChannel')
+    nc = q.expect('dbus-signal', signal='NewChannel',
+            predicate=lambda e: cs.CHANNEL_TYPE_CONTACT_LIST not in e.args)
     path, ct, ht, h, sh = nc.args
     assert ct == cs.CHANNEL_TYPE_STREAMED_MEDIA, ct
     assert ht == cs.HT_CONTACT, ht
@@ -74,7 +86,7 @@ def test(q, bus, conn, stream):
 
     # peer gets the transport
     e = q.expect('stream-iq', predicate=jp.action_predicate('transport-info'))
-    assertEquals(remote_jid, e.query['initiator'])
+    assertEquals(jt2.peer, e.query['initiator'])
 
     stream.send(make_result_iq(stream, e.stanza))
 
@@ -108,11 +120,11 @@ def test(q, bus, conn, stream):
     # Foo would like to gaze upon our beautiful complexion
     node = jp.SetIq(jt2.peer, jt2.jid, [
         jp.Jingle(jt2.sid, jt2.peer, 'content-add', [
-            jp.Content('videostream', 'initiator', 'both', [
+            jp.Content('videostream', 'initiator', 'both',
                 jp.Description('video', [
-                    jp.PayloadType(name, str(rate), str(id)) for
-                        (name, id, rate) in jt2.video_codecs ]),
-            jp.TransportGoogleP2P() ]) ]) ])
+                    jp.PayloadType(name, str(rate), str(id), parameters) for
+                        (name, id, rate, parameters) in jt2.video_codecs ]),
+            jp.TransportGoogleP2P()) ]) ])
     stream.send(jp.xml(node))
 
     added, nsh = q.expect_many(
@@ -149,7 +161,7 @@ def test(q, bus, conn, stream):
         # It's not entirely clear that this *needs* to fire here...
         EventPattern('dbus-signal', signal='SetStreamSending', args=[False]),
         )
-    assertEquals(remote_jid, ti.query['initiator'])
+    assertEquals(jt2.peer, ti.query['initiator'])
 
     stream.send(make_result_iq(stream, e.stanza))
 
@@ -183,3 +195,4 @@ def test(q, bus, conn, stream):
 
 if __name__ == '__main__':
     exec_test(test)
+    exec_test(test_bare_jid)
