@@ -12,10 +12,6 @@ from servicetest import EventPattern
 import constants as cs
 
 def test(q, bus, conn, stream):
-    conn.Connect()
-    q.expect('dbus-signal', signal='StatusChanged',
-            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
-
     m = domish.Element((None, 'message'))
     m['from'] = 'foo@bar.com'
     m['type'] = 'chat'
